##########################
# Raspberry Pi Pico
# ST7789 LCD
# AMG8833 Thermography
# Display Program
##########################

import math
import time
import board
import busio
import terminalio
import displayio
import digitalio
from adafruit_display_text import label
from adafruit_st7789 import ST7789
import adafruit_amg88xx
from digitalio import DigitalInOut, Pull

i2c = busio.I2C(board.GP1, board.GP0)
amg = adafruit_amg88xx.AMG88XX(i2c)

MINTEMP = 5
MAXTEMP = 50

COLORDEPTH = 256

FONTSCALE = 2
TEXT_COLOR = 0xFFFFFF
BG_COLOR =0x000000

displayio.release_displays()

# for waveshare LCD
#spi = busio.SPI(clock=board.GP10, MOSI=board.GP11)
#tft_cs = board.GP9
#tft_dc = board.GP8
#rst = digitalio.DigitalInOut(board.GP12)
#rst.direction = digitalio.Direction.OUTPUT
#rst.value = True
#button1 = DigitalInOut(board.GP15)
#button2 = DigitalInOut(board.GP17)

#for PIMORONI LCD
spi = busio.SPI(clock=board.GP18, MOSI=board.GP19)
tft_cs = board.GP17
tft_dc = board.GP16
button1 = DigitalInOut(board.GP14)
button2 = DigitalInOut(board.GP15)

button1.pull = Pull.UP
button2.pull = Pull.UP

display_bus = displayio.FourWire(spi, command=tft_dc, chip_select=tft_cs)
display = ST7789(
    display_bus, rotation=270, width=240, height=135, rowstart=40, colstart=53
)

def map_value(x, in_min, in_max, out_min, out_max):
    if x > in_max:
        x = in_max
    if x < in_min:
        x = in_max
    return int((x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min)

# Make the display context
all_group = displayio.Group(scale=1)
temp_group = displayio.Group(scale=16)

color_bitmap = displayio.Bitmap(8, 8, 256)
scale_bitmap = displayio.Bitmap(16,128,256)
color_palette = displayio.Palette(256)
temp_sprite = displayio.TileGrid(color_bitmap, pixel_shader=color_palette, x=0, y=0)
scale_sprite = displayio.TileGrid(scale_bitmap, pixel_shader=color_palette, x=128, y=0)

temp_group.append(temp_sprite)
all_group.append(scale_sprite)
all_group.append(temp_group)
display.show(all_group)

#Genarate Color Pallette
for i in range(64):
    color_palette[255-i] = (255,i*4,0)
    color_palette[255-(i+64)] = (0,255,i*4)
    color_palette[255-(i+128)] = (0,255-i*4,255)
    color_palette[255-(i+192)] = (0,0,255-i*4)
color_palette[0] = (0,0,0)

#Genarate Heat Map Scale image
for i in range(128):
    for j in range(16):
        scale_bitmap[j,i] = i*2

def text_update():
    text_area1 = label.Label(terminalio.FONT, text=str(MINTEMP)+"   ", color=TEXT_COLOR,background_color=BG_COLOR)
    text_width = 90
    text_group1 = displayio.Group(scale=FONTSCALE, x=150, y=10,)
    text_group1.append(text_area1) 
    all_group.append(text_group1)

    text_area2 = label.Label(terminalio.FONT, text=str(MAXTEMP)+"   ", color=TEXT_COLOR,background_color=BG_COLOR)
    text_width = 90
    text_group2 = displayio.Group(scale=FONTSCALE, x=150, y=120,)
    text_group2.append(text_area2) 
    all_group.append(text_group2)

text_update()
time.sleep(0.1)

while True:

    # read the pixels
    pixels = []
    for row in amg.pixels:
        pixels = pixels + row
    npixels = [map_value(p, MINTEMP, MAXTEMP, 0, COLORDEPTH - 1) for p in pixels]

    # draw everything
    for ix in range(8):
        for iy in range(8):
            color_bitmap[7-iy,7-ix] = npixels[iy+ix*8]

    if button1.value == False :
        MINTEMP = MINTEMP + 5
        if MINTEMP == MAXTEMP:
            MINTEMP = 0
        text_update()
            
    if button2.value == False :
        MAXTEMP = MAXTEMP + 5
        if MAXTEMP > 80.0 :
            MAXTEMP = MINTEMP+5
        text_update()
                
